package com.qs.mp.web.controller.api.user;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.github.xiaoymin.knife4j.annotations.DynamicParameter;
import com.github.xiaoymin.knife4j.annotations.DynamicResponseParameters;
import com.qs.mp.admin.domain.Ticket;
import com.qs.mp.admin.domain.TicketAwardsPrize;
import com.qs.mp.admin.domain.TicketBox;
import com.qs.mp.admin.domain.TicketPackage;
import com.qs.mp.admin.domain.param.TicketBoxParam;
import com.qs.mp.admin.domain.param.TicketParam;
import com.qs.mp.admin.domain.vo.TicketBoxListVO;
import com.qs.mp.admin.domain.vo.TicketBoxVO;
import com.qs.mp.admin.domain.vo.TicketListVO;
import com.qs.mp.admin.domain.vo.TicketVO;
import com.qs.mp.admin.service.ITicketAwardsPrizeService;
import com.qs.mp.admin.service.ITicketBoxService;
import com.qs.mp.admin.service.ITicketPackageService;
import com.qs.mp.admin.service.ITicketService;
import com.qs.mp.common.core.domain.AjaxResult;
import com.qs.mp.common.core.page.TableDataInfo;
import com.qs.mp.common.enums.*;
import com.qs.mp.common.utils.LogUtil;
import com.qs.mp.common.utils.StringUtils;
import com.qs.mp.framework.security.handle.HostHolder;
import com.qs.mp.user.domain.UserHitPrize;
import com.qs.mp.user.domain.UserTicketOrderItem;
import com.qs.mp.user.domain.vo.TicketHitPrizeBarrageVO;
import com.qs.mp.user.service.IUserHitPrizeService;
import com.qs.mp.user.service.IUserTicketOrderItemService;
import com.qs.mp.utils.SecurityUtils;
import com.qs.mp.web.controller.common.BaseApiController;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.AllArgsConstructor;
import ma.glasnost.orika.MapperFacade;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.POST;


@RestController
@RequestMapping("/api/v1/mp/user")
@Api(tags = "盲票商城接口")
@AllArgsConstructor
public class UserTicketController extends BaseApiController {


  @Autowired
  private ITicketBoxService ticketBoxService;

  @Autowired
  private ITicketAwardsPrizeService ticketAwardsPrizeService;

  @Autowired
  private ITicketService ticketService;

  @Autowired
  private IUserTicketOrderItemService userTicketOrderItemService;

  @Autowired
  private IUserHitPrizeService userHitPrizeService;

  @Autowired
  private ITicketPackageService ticketPackageService;

  @Autowired
  private MapperFacade mapperFacade;


  @Autowired
  private HostHolder hostHolder;

  /**
   * 盲票进货列表
   */
  @PostMapping("/mall/ticket/list")
  @ApiOperation(value = "盲票组列表", notes = "获取所有盲票信息")
  public TableDataInfo list(@RequestBody TicketBoxParam param) {
    // TODO 根据标签过滤
    startPage();
    List<TicketBox> ticketBoxes = ticketBoxService.list(
        new LambdaQueryWrapper<TicketBox>().eq(TicketBox::getType, param.getType())
            .eq(null != param.getCategoryId() && 0 != param.getCategoryId(),
                TicketBox::getCategoryId, param.getCategoryId())
            .eq(TicketBox::getStatus, TicketBoxStatusEnum.PUT_ON)
            .orderByDesc(TicketBox::getSortWeight)
    );
    List<TicketBoxListVO> ticketBoxListVOList = mapperFacade.mapAsList(ticketBoxes,
        TicketBoxListVO.class);
    TableDataInfo rspData = getDataTable(ticketBoxes);
    rspData.setRows(ticketBoxListVOList);
    return rspData;
  }


  /**
   * 查看盲票详情
   */
  @PostMapping("/mall/ticket/detail")
  @ApiOperation(value = "查看盲票详情", notes = "根据盲票组ID，获取盲票信息")
  @ApiResponses(
          @ApiResponse(code = 200, message = "成功", response = TicketBoxVO.class)
  )
  public AjaxResult getInfo(@RequestBody TicketBoxParam param) {
    if (StringUtils.isBlank(param.getBoxId())) {
      return AjaxResult.error("参数异常，盲票组ID缺失");
    }
    TicketBox ticketBox = ticketBoxService.getById(param.getBoxId());
    TicketBoxVO ticketBoxVO = mapperFacade.map(ticketBox, TicketBoxVO.class);
    ticketBoxVO.setPrizeList(
        ticketAwardsPrizeService.listPrizeVO(new QueryWrapper<TicketAwardsPrize>()
            .eq("t1.box_id", ticketBox.getBoxId()).orderByAsc("t1.sort").orderByDesc("t2.value")));
    return AjaxResult.success(ticketBoxVO);
  }

  /**
   * 扫码查看盲票幸运数字
   */
  @PostMapping("/ticket/queryLuckyNum")
  @ApiOperation(value = "查看盲票幸运数字", notes = "根据盲票组ID，获取盲票幸运数字")
  public AjaxResult queryLuckyNum(@RequestBody TicketParam param) {

    if (StringUtils.isBlank(param.getSerialNo())) {
      return AjaxResult.error("参数异常，盲票序列号缺失");
    }

    Ticket ticket = ticketService.getOne(
        new LambdaQueryWrapper<Ticket>().eq(Ticket::getSerialNo, param.getSerialNo()));
    if (null == ticket) {
      return AjaxResult.error("参数异常，盲票不存在");
    }

    /*if (ticket.getStatus() == TicketStatusEnum.CASHED) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1017);
    }*/
    TicketPackage ticketPackage = ticketPackageService.getById(ticket.getPkgId());
    if (ticketPackage.getStatus() != TicketPkgStatusEnum.SOLD) {
      LogUtil.error(logger, "盲票未激活。sn:{0},pkgId:{1}", new Object[]{ticket.getSerialNo(), ticket.getPkgId()});
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1023);
    }

    TicketBox ticketBox = ticketBoxService.getById(ticket.getBoxId());

    TicketVO ticketVO = new TicketVO();
    ticketVO.setPrizeList(ticketAwardsPrizeService.listPrizeVO(new QueryWrapper<TicketAwardsPrize>()
        .eq("t1.box_id", ticketBox.getBoxId()).orderByAsc("t1.sort").orderByDesc("t2.value")));
    ticketVO.setTicketId(ticket.getTicketId());
    ticketVO.setBoxId(ticket.getBoxId());
    ticketVO.setTitle(ticketBox.getTitle());
    ticketVO.setSerialNo(ticket.getSerialNo());
    ticketVO.setFacePrice(ticket.getFacePrice());
    ticketVO.setSalePrice(ticketBox.getSalePrice());
    ticketVO.setPlainLuckyNum(ticket.getPlainLuckyNum());
    ticketVO.setStatus(ticket.getStatus().getValue());
    ticketVO.setPicUrl(ticketBox.getPicUrl());

    return AjaxResult.success(ticketVO);
  }

  /**
   * 根据盲票购买订单号查看盲票详情
   */
  @PostMapping("/ticket/query")
  @ApiOperation(value = "查看盲票详情", notes = "根据盲票购买订单号，查看盲票详情")
  public AjaxResult queryTicket(@RequestBody TicketParam param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    if (StringUtils.isBlank(param.getOrderId())) {
      return AjaxResult.error("参数缺失");
    }

    TicketListVO ticketListVO = userTicketOrderItemService.queryTicketVO(userId,
        param.getOrderId());
    return AjaxResult.success(ticketListVO);
  }

  /**
   * 查看能兑奖的奖品
   */
  @PostMapping("/ticket/queryHitPrizeList")
  @ApiOperation(value = "查看兑奖奖品", notes = "根据盲票ID，查看兑奖奖品")
  public AjaxResult queryHitPrizeList(@RequestBody TicketParam param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    if (StringUtils.isBlank(param.getTicketId())) {
      return AjaxResult.error("参数缺失");
    }
    Ticket ticket = ticketService.getById(param.getTicketId());

    if (null == ticket) {
      return AjaxResult.error("参数异常，盲票不存在");
    }
    if (ticket.getStatus() == TicketStatusEnum.CASHED) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1017);
    }
    if (ticket.getStatus() != TicketStatusEnum.ACTIVATED) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1019);
    }
    // 已付款的盲票，需要校验当前用户是否有权限查看
    UserTicketOrderItem orderItem = userTicketOrderItemService.queryFinishedOrderItem(userId,
        ticket.getTicketId());
    if (null == orderItem) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1018);
    }

    List<TicketAwardsPrize> ticketAwardsPrizes = userHitPrizeService.listPrize(ticket, userId);
    JSONObject jsonObject = new JSONObject();
    jsonObject.put("prizeList", ticketAwardsPrizes);
    jsonObject.put("ticketId", ticket.getTicketId());
    jsonObject.put("boxId", ticket.getBoxId());
    return AjaxResult.success(jsonObject);
  }

  /**
   * 兑奖
   */
  @PostMapping("/ticket/cashPrize")
  @ApiOperation(value = "兑奖", notes = "选择奖品兑奖")
  public AjaxResult cashPrize(@RequestBody TicketParam param) {
    Long userId = SecurityUtils.getLoginUser().getUserId();
    if (StringUtils.isBlank(param.getTicketId()) || StringUtils.isBlank(param.getAwardsId())
        || StringUtils.isBlank(param.getPrizeId())) {
      return AjaxResult.error("参数缺失");
    }

    Ticket ticket = ticketService.getById(param.getTicketId());
    if (null == ticket) {
      return AjaxResult.error("参数异常，盲票不存在");
    }
    if (ticket.getStatus() == TicketStatusEnum.CASHED) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1017);
    }
    if (ticket.getStatus() != TicketStatusEnum.ACTIVATED) {
      return AjaxResult.error(ErrorCodeEnum.ERROR_CODE_1019);
    }

    userHitPrizeService.cashPrize(ticket, userId, param.getAwardsId(), param.getPrizeId());
    return AjaxResult.success();
  }

  @PostMapping("/ticket/hitPrizeBarrage")
  @ApiOperation(value = "中奖弹幕", notes = "获取最近中奖的 20-22 条信息")
  @DynamicResponseParameters(properties = {
    @DynamicParameter(name = "listOne", value = "列表 1",required = true, dataTypeClass = TicketHitPrizeBarrageVO.class),
    @DynamicParameter(name = "listTwo", value = "列表 2",required = true, dataTypeClass = TicketHitPrizeBarrageVO.class)
  })
  public AjaxResult hitPrizeBarrage() {

    // 获取用户 id
    Long userId = 0L;

    if (hostHolder.getUser() != null) {
      userId = hostHolder.getUser().getUserId();
    }

    // 弹幕列表
    List<TicketHitPrizeBarrageVO> listOne = new ArrayList<>();
    List<TicketHitPrizeBarrageVO> listTwo = new ArrayList<>();

    // 获取用户当天中奖的 2 条弹幕
    if (userId != 0L) {
      List<TicketHitPrizeBarrageVO> ticketHitPrizeBarrageVOList = userHitPrizeService.hitPrizeBarrageByUserId(userId);
      if (CollectionUtils.isNotEmpty(ticketHitPrizeBarrageVOList)) {
        for (int i = 0; i < ticketHitPrizeBarrageVOList.size(); i++) {
          // 类型为盲豆则拼接数量
          if(TicketPrizeTypeEnum.COIN.getValue().equals(ticketHitPrizeBarrageVOList.get(i).getPrizeType())){
            ticketHitPrizeBarrageVOList.get(i).setPrizeInfo(ticketHitPrizeBarrageVOList.get(i).getPrizeInfo() + ticketHitPrizeBarrageVOList.get(i).getValue() + "颗");
          }

          if (i % 2 == 0) {
            listOne.add(ticketHitPrizeBarrageVOList.get(i));
          }else {
            listTwo.add(ticketHitPrizeBarrageVOList.get(i));
          }
        }
      }
    }
    // 获取除该用户外，最近中奖的 40 条弹幕信息
    List<TicketHitPrizeBarrageVO> ticketHitPrizeBarrageVOList = userHitPrizeService.hitPrizeBarrage(userId);

    // 获取除该用户外，近 6 个月，8 条高级中奖弹幕信息
    List<TicketHitPrizeBarrageVO> expensiveHitPrizeBarrageList = userHitPrizeService.expensiveHitPrizeBarrage(userId);
    int count = 0;
    int index = 0;
    if (CollectionUtils.isNotEmpty(expensiveHitPrizeBarrageList)) {
      count = expensiveHitPrizeBarrageList.size();
      // 去重
      ticketHitPrizeBarrageVOList.removeAll(expensiveHitPrizeBarrageList);
    }


    for (int i = 0; i < ticketHitPrizeBarrageVOList.size(); i++) {
      // 类型为盲豆则拼接数量
      if(TicketPrizeTypeEnum.COIN.getValue().equals(ticketHitPrizeBarrageVOList.get(i).getPrizeType())){
        ticketHitPrizeBarrageVOList.get(i).setPrizeInfo(ticketHitPrizeBarrageVOList.get(i).getPrizeInfo() + ticketHitPrizeBarrageVOList.get(i).getValue() + "颗");
      }

      // 插入大奖弹幕
      if (count != 0 && i % 5 == 0) {
        if (index % 2 == 0) {
          listOne.add(expensiveHitPrizeBarrageList.get(index));
        }else {
          listTwo.add(expensiveHitPrizeBarrageList.get(index));
        }
        count--;
        index++;
      }

      if (i % 2 == 0) {
        listOne.add(ticketHitPrizeBarrageVOList.get(i));
      }else {
        listTwo.add(ticketHitPrizeBarrageVOList.get(i));
      }
    }
    Map<String,List<TicketHitPrizeBarrageVO>> ticketHitPrizeBarrageMap = new HashMap<>();
    ticketHitPrizeBarrageMap.put("listOne",listOne);
    ticketHitPrizeBarrageMap.put("listTwo",listTwo);

    return AjaxResult.success(ticketHitPrizeBarrageMap);
  }






}
